@@ -1,11 +1,21 @@
 import re
+import copy
 import inspect
 from types import MethodType
 from typing import TypeVar, Dict
-from .helpers import get_url, render_replace
+
+from django.forms.widgets import Select
+
+from .helpers import get_url, render_replace, DUMMY_ID
+
 KT = TypeVar('KT')
 VT = TypeVar('VT')
 
+EDIT_SEND = 'django_datatables.row_send(this)'
+MAKE_EDIT = 'django_datatables.make_edit(this)'
+PENCIL = '<i class="cell-edit"></i>'
+EDIT_CELL_HTML = f'<div class="cell-div" tabindex="0" onfocus="{MAKE_EDIT}" onclick="$(this).focus()">%1%{PENCIL}</div>'
+
 
 class ColumnNameError(Exception):
     pass
@@ -17,6 +27,15 @@ class DatatableColumnError(Exception):
 
 class ColumnBase:
 
+    """
+    kwargs
+        * **enabled** not included in table columns
+        * **hidden** not displayed by default may be overridden by table saving options
+    """
+
+    popover_html = ('<button type="button" class="ml-1 btn btn-link p-0" data-html=true data-placement="top" '
+                    'data-toggle="popover" data-content="{}"><i class="far fa-question-circle"></i></button>')
+
     @staticmethod
     def merge_kwargs_locals(local_vars):
         kwargs = local_vars.pop('kwargs', {})
@@ -59,9 +78,13 @@ class ColumnBase:
         self.column_name, options = self.extract_options(kwargs.get('column_name', ''))
         self.options: Dict[KT, VT] = options
         self.model_path = kwargs.pop('model_path', None)
-        if not self.model_path:
+        if not self.model_path is not None:
             self.model_path = self.get_model_path(self.column_name)
+            if '/' in self.model_path:
+                self.model_path = self.model_path[self.model_path.find('/') + 1:]
         self.model = kwargs.pop('model', None)
+        self.table = kwargs.pop('table', None)
+        self.enabled = kwargs.pop('enabled', True)
         if kwargs.pop('column_name', None) == field:
             self.field = self.column_name
         else:
@@ -70,13 +93,54 @@ class ColumnBase:
         self.title = self.title_from_name(self.column_name)
         self.column_type = 0
         self._annotations = None
+        self._annotations_value = None
+        self._aggregations = None
         self.additional_columns = []
         self.kwargs = kwargs
         self.replace_list = []
         self.blank = None
+        self.popover = None
+        self.setup_kwargs(kwargs)
+        self.result_processes = {}
+        self.col_setup()
+        self.edit_type = None
+
+        if self.table and self.column_name in getattr(self.table, 'edit_fields', []):
+            self.setup_edit()
+
         if not hasattr(self, 'row_result'):
             self.row_result = MethodType(self.__row_result, self)
-        self.setup_kwargs(kwargs)
+
+    def dropdown_edit(self, choices):
+        options = self.table.edit_options.get(self.column_name, {})
+        attrs = {} if options.get('select2') else {'onchange': '$(this).blur()', 'onfocusout': f'{EDIT_SEND}'}
+        return {'render': [render_replace(column=self.column_name + ':1', html=EDIT_CELL_HTML)],
+                'field_array': True,
+                'input_html': Select(choices=choices).render('', '', attrs=attrs),
+                'edit_options': options}
+
+    def setup_edit(self):
+        if '__' in self.field:
+            if self.field.count('__') > 1:
+                raise DatatableColumnError('Cannot edit this field')
+            self.edit_type = 'FK'
+            field = self.field[self.field.find('__') + 2:]
+            choices = list(self.model.objects.values_list('id', field))
+            self.field = ['id', field]
+            self.options.update(self.dropdown_edit(choices))
+        else:
+            self.options['render'] = [render_replace(column=self.column_name, html=EDIT_CELL_HTML)]
+            self.options['input_html'] = f'<input class="cell-input" onfocusout="{EDIT_SEND}" type="text">'
+
+    def alter_object(self, row_object, value):
+        if self.edit_type == 'FK':
+            setattr(row_object, self.field[0][:self.field[0].find('__')] + '_id', value[0])
+        else:
+            setattr(row_object, self.field, value)
+        row_object.save()
+
+    def col_setup(self):
+        pass
 
     @property
     def annotations(self):
@@ -84,7 +148,26 @@ class ColumnBase:
 
     @annotations.setter
     def annotations(self, value):
-        for f in value:
+        self._annotations = self._set_annotations(value)
+
+    @property
+    def annotations_value(self):
+        return self._annotations_value
+
+    @annotations_value.setter
+    def annotations_value(self, value):
+        self._annotations_value = self._set_annotations(value)
+
+    def _set_annotations(self, value):
+        annotations = copy.deepcopy(value)
+        if self.model_path:
+            new_annotations = {}
+            for k in annotations:
+                new_annotations[self.model_path + k] = annotations[k]
+                for e in new_annotations[self.model_path + k].source_expressions:
+                    e.name = self.model_path + e.name
+            annotations = new_annotations
+        for f in annotations:
             if self.field is None:
                 self._field = f
             elif type(self.field) == str:
@@ -93,7 +176,22 @@ class ColumnBase:
             else:
                 if f not in self.field:
                     self._field.append(f)
-        self._annotations = value
+        return annotations
+
+    def get_annotations(self, **_kwargs):
+        return self._annotations
+
+    @property
+    def aggregations(self):
+        return self._aggregations
+
+    @aggregations.setter
+    def aggregations(self, value):
+        self._aggregations = self._set_aggregations(value)
+
+    def _set_aggregations(self, value):
+        aggregations = copy.deepcopy(value)
+        return aggregations
 
     @property
     def column_name(self):
@@ -182,7 +280,7 @@ class ColumnBase:
     @staticmethod
     def title_from_name(name):
         if type(name) == str and len(name) > 0:
-            field_no_path = name.split('__')[-1]
+            field_no_path = name.split('/')[-1].split('__')[-1]
             if field_no_path.find('_') > 0:
                 return field_no_path.replace('_', ' ').title()
             else:
@@ -214,10 +312,6 @@ class DatatableColumn(ColumnBase):
         if not self.initialise(locals()):
             return
         super().__init__(**kwargs)
-        self.col_setup()
-
-    def col_setup(self):
-        pass
 
 
 class LambdaColumn(ColumnBase):
@@ -250,6 +344,9 @@ class TextFieldColumn(ColumnBase):
 
 class ColumnLink(ColumnBase):
 
+    base_link_html = '%1%'
+    base_link_css = None
+
     @property
     def url(self):
         return self._url
@@ -258,26 +355,30 @@ class ColumnLink(ColumnBase):
     def url(self, url_name):
         self._url = get_url(url_name)
 
-    def __init__(self, *, url_name, link_ref_column='id', link_html='%1%', var='%1%', **kwargs):
+    def __init__(self, *, url_name, link_ref_column='id', link_html=None, link_css='', var='%1%', **kwargs):
         if not self.initialise(locals()):
             return
         super().__init__(**self.kwargs)
+        link_ref_column = self.model_path + link_ref_column
         self.url = url_name
 
+        if not link_html:
+            link_html = self.base_link_html
+
+        if not link_css:
+            link_css = self.base_link_css
+        link_css = f' class="{link_css}"' if link_css else ''
+        link = f'<a{link_css} href="{self.url}">{{}}</a>'
         if isinstance(self.field, (list, tuple)):
-            self.options['render'] = [render_replace(column=self.column_name + ':0',
-                                                     html=f'<a href="{self.url}">%1%</a>',
-                                                     var='999999'),
-                                      render_replace(column=self.column_name + ':1'),
-                                      ]
+            self.options['render'] = [
+                render_replace(column=self.column_name + ':0', html=link.format('%1%'), var='999999'),
+                render_replace(column=self.column_name + ':1'),
+            ]
         elif var not in link_html:
-            self.options['render'] = [render_replace(column=link_ref_column,
-                                                     html=f'<a href="{self.url}">{link_html}</a>',
-                                                     var='999999')]
+            self.options['render'] = [render_replace(column=link_ref_column, html=link.format(link_html), var='999999')]
         else:
-            self.options['render'] = [render_replace(column=self.column_name,
-                                                     html=f'<a href="{self.url}">{link_html}</a>',
-                                                     var=var), render_replace(column=link_ref_column, var='999999')]
+            self.options['render'] = [render_replace(column=self.column_name, html=link.format(link_html), var=var),
+                                      render_replace(column=link_ref_column, var='999999')]
 
 
 class ManyToManyColumn(DatatableColumn):
@@ -356,11 +457,23 @@ class ChoiceColumn(ColumnBase):
 
     def setup_kwargs(self, kwarg_dict):
         choices = kwarg_dict.pop('choices')
-        self.options = {c[0]: c[1] for c in choices}
+        self.choices = {c[0]: c[1] for c in choices}
         super().setup_kwargs(kwarg_dict)
 
     def row_result(self, data, _page_data):
-        return self.options.get(data[self.field], '')
+        return self.choices.get(data[self.field], '')
+
+    def setup_edit(self):
+        choices = list(self.choices.items())
+        self.options.update(self.dropdown_edit(choices))
+        self.row_result = self.edit_row_result
+
+    def edit_row_result(self, data, _page_data):
+        return [data[self.field], self.choices.get(data[self.field], '')]
+
+    def alter_object(self, row_object, value):
+        setattr(row_object, self.field, value[0])
+        row_object.save()
 
 
 class CurrencyPenceColumn(ColumnBase):
@@ -368,7 +481,7 @@ class CurrencyPenceColumn(ColumnBase):
     def row_result(self, data, _page_data):
         try:
             return '{:.2f}'.format(data[self.field] / 100.0)
-        except KeyError:
+        except (KeyError, TypeError):
             return
 
     def setup_kwargs(self, kwargs):
@@ -381,7 +494,7 @@ class CurrencyColumn(ColumnBase):
     def row_result(self, data, _page_data):
         try:
             return '{:.2f}'.format(data[self.field])
-        except KeyError:
+        except (KeyError, TypeError):
             return
 
     def setup_kwargs(self, kwargs):
@@ -436,3 +549,27 @@ class CallableColumn(DatatableColumn):
         super().setup_kwargs(kwargs)
         self.object_function = getattr(self.model, self.field[len(self.model_path):])
         self.field = kwargs.get('parameters')
+
+
+class NoHeadingColumn(DatatableColumn):
+    def __init__(self, **kwargs):
+        kwargs['title'] = ''
+        kwargs['no_col_search'] = True
+        kwargs['column_defs'] = {'orderable': False}
+        super().__init__(**kwargs)
+
+
+class MenuColumn(NoHeadingColumn):
+    """
+    This is used to render a django-menus in a column.
+    example:
+
+    MenuColumn(column_name='menu', field='id', menu=HtmlMenu(self.request, 'button_group').add_items(
+                    ('view 1', 'View 1', {'url_kwargs': {'int': DUMMY_ID}}),
+                    ('view 2', 'View 2', {'url_kwargs': {'int': DUMMY_ID}})
+                )),
+    """
+    def __init__(self, menu, **kwargs):
+        menu_rendered = menu.render().replace(str(DUMMY_ID), '%1%')
+        kwargs['render'] = [render_replace(html=menu_rendered, column=kwargs['column_name'])]
+        super().__init__(**kwargs)
