@@ -1,7 +1,9 @@
 import json
 from inspect import isclass
 from typing import TypeVar, Dict
+from ajax_helpers.utils import random_string
 from django.db import models
+from django.db.models import Sum
 from django.views.generic import TemplateView
 from django.http import HttpResponse, JsonResponse
 from django.template.loader import render_to_string
@@ -10,7 +12,7 @@ from .detect_device import detect_device
 from .columns import ColumnBase, DateColumn, ChoiceColumn, BooleanColumn, CallableColumn
 from .model_def import DatatableModel
 from .filters import DatatableFilter
-
+from .models import SavedState
 KT = TypeVar('KT')
 VT = TypeVar('VT')
 
@@ -19,6 +21,10 @@ class DatatableError(Exception):
     pass
 
 
+class DatatableExcludedRow(Exception):
+    pass
+
+
 class ColumnInitialisor:
 
     def __init__(self, start_model, path, field_prefix='', name_prefix='', **kwargs):
@@ -31,9 +37,9 @@ class ColumnInitialisor:
             path = path[0]
         self.kwargs = kwargs
         self.columns = []
-        self.kwargs['model_path'] = field_prefix
         self.callable = False
-
+        self.next_prefix = ''
+        explicit_name = False
         if isclass(path):
             self.setup = path
             self.kwargs['column_name'] = path.__name__
@@ -43,10 +49,14 @@ class ColumnInitialisor:
                 self.kwargs['column_name'] = type(path).__name__
         elif isinstance(path, str):
             self.path, options = ColumnBase.extract_options(path)
+            self.path = field_prefix + self.path
             self.kwargs.update(options)
             if start_model is not None:
                 self.model, self.django_field, self.setup = DatatableModel.get_setup_data(start_model, self.path)
-            self.kwargs['column_name'] = path
+            if 'column_name' in self.kwargs:
+                explicit_name = True
+            else:
+                self.kwargs['column_name'] = path
 
             if '__' in self.path:
                 split_path = self.path.split('__')
@@ -54,24 +64,27 @@ class ColumnInitialisor:
                 self.next_prefix = '__'.join(split_path[:-1]) + '__'
             else:
                 self.field = self.path
-                self.next_prefix = ''
             self.callable = callable(getattr(self.model, self.field, None))
         else:
             raise DatatableError('Unknown type in columns ' + str(path))
 
-        if 'column_name' in kwargs and name_prefix:
-            self.kwargs['column_name'] = f'{name_prefix}/{self.kwargs["column_name"]}'
+        if 'column_name' in kwargs and name_prefix and not explicit_name:
+            self.kwargs['column_name'] = f'{name_prefix}/{field_prefix}{self.kwargs["column_name"]}'
 
     def get_columns(self):
         self.kwargs['model'] = self.model
+        self.kwargs['model_path'] = self.next_prefix
         if isclass(self.setup):
             self.columns.append(self.setup(**self.kwargs))
         elif isinstance(self.setup, ColumnBase):
             if self.setup.initialised:
+                if 'table' in self.kwargs:
+                    self.setup.table = self.kwargs['table']
                 self.columns.append(self.setup)
             else:
                 self.columns.append(self.setup.get_class_instance(**self.kwargs))
         elif isinstance(self.setup, list):
+            del self.kwargs['column_name']
             for c in self.setup:
                 self.columns += ColumnInitialisor(start_model=self.start_model, path=c, field_prefix=self.next_prefix,
                                                   name_prefix=self.field, **self.kwargs).get_columns()
@@ -92,31 +105,40 @@ class ColumnInitialisor:
     def add_django_field_column(self):
         if 'title' not in self.kwargs:
             self.kwargs['title'] = self.django_field.verbose_name.title()
-        field_type = type(self.django_field)
-        if field_type in [models.DateField, models.DateTimeField]:
+        if isinstance(self.django_field, (models.DateField, models.DateTimeField)):
             self.columns.append(DateColumn(**self.kwargs))
-        elif (field_type in [models.IntegerField, models.PositiveSmallIntegerField, models.PositiveIntegerField]
+        elif (isinstance(self.django_field,
+                         (models.IntegerField, models.PositiveSmallIntegerField, models.PositiveIntegerField))
               and self.django_field.choices is not None and len(self.django_field.choices) > 0):
             self.columns.append(ChoiceColumn(choices=self.django_field.choices, **self.kwargs))
-        elif field_type == models.BooleanField:
+        elif isinstance(self.django_field, models.BooleanField):
             self.columns.append(BooleanColumn(**self.kwargs))
         else:
             self.columns.append(ColumnBase(**self.kwargs))
 
 
 class DatatableTable:
+    datatable_template = 'datatables/table.html'
 
-    def __init__(self, table_id, model=None, table_options=None, table_classes=None):
+    edit_options = {}
+    edit_fields = []
+
+    def __init__(self, table_id=None, model=None, table_options=None, table_classes=None, view=None, **kwargs):
         self.columns = []
-        self.table_id = table_id
+        self.table_id = table_id if table_id else random_string()
         self.table_options: Dict[KT, VT] = {'pageLength': 100}
+        self.view = view
         if table_options:
             self.table_options.update(table_options)
 
         self.ajax_data = True
+        self.table_data = None
         self.model = model
+        self.page_results = {}
 
         # django query attributes
+        self.initial_filter = {}
+        self.initial_values = []
         self.filter = {}
         self.exclude = {}
         self.distinct = None
@@ -127,7 +149,8 @@ class DatatableTable:
         self.js_filter_list = []
         self.plugins = []
         self.omit_columns = []
-        self.datatable_template = 'datatables/table.html'
+
+        self.kwargs = kwargs
 
         if table_classes:
             self.table_classes = table_classes
@@ -140,11 +163,17 @@ class DatatableTable:
     def column(self, column_name):
         return self.find_column(column_name)[0]
 
-    def add_js_filters(self, name_or_template, column_ids, **kwargs):
-        if type(column_ids) == str:
-            column_ids = [column_ids]
-        for c in column_ids:
-            self.js_filter_list.append(DatatableFilter(name_or_template, self, column=self.find_column(c)[0], **kwargs))
+    def add_js_filters(self, name_or_template, *column_ids, filter_class=DatatableFilter, **kwargs):
+        if isclass(name_or_template):
+            filter_class = name_or_template
+        if column_ids:
+            if isinstance(column_ids[0], (list, tuple)):
+                column_ids = column_ids[0]
+            for c in column_ids:
+                self.js_filter_list.append(filter_class(name_or_template, self, column=self.find_column(c)[0],
+                                                        **kwargs))
+        else:
+            self.js_filter_list.append(filter_class(name_or_template, self, **kwargs))
 
     def js_filter(self, name_or_template, column_id, **kwargs):
         return DatatableFilter(name_or_template, self, column=self.find_column(column_id)[0], **kwargs)
@@ -156,13 +185,31 @@ class DatatableTable:
     def extra_filters(query):
         return query
 
+    @staticmethod
+    def view_filter(query, table):
+        return query
+
     def get_query(self, **_kwargs):
         annotations = {}
+        annotations_value = {}
+        aggregations = {}
         for c in self.columns:
-            if c.annotations:
-                annotations.update(c.annotations)
-        query = (self.model.objects.annotate(**annotations).exclude(**self.exclude).values(*self.fields())
-                 .order_by(*self.order_by))
+            if c.get_annotations(**self.kwargs):
+                annotations.update(c.get_annotations(**self.kwargs))
+            if c.annotations_value:
+                annotations_value.update(c.annotations_value)
+            if c.aggregations:
+                aggregations.update(c.aggregations)
+        query = self.model.objects
+        # Use initial values to group_by for annotations
+        if self.initial_values:
+            query = query.values(*self.initial_values)
+        if self.initial_filter:
+            query = (query.filter(self.initial_filter) if isinstance(self.initial_filter, models.Q)
+                     else query.filter(**self.initial_filter))
+        if annotations_value:
+            query = query.annotate(**annotations_value).values(*[f for f in annotations_value])
+        query = query.annotate(**annotations).exclude(**self.exclude).values(*self.fields()).order_by(*self.order_by)
         if isinstance(self.filter, models.Q):
             query = query.filter(self.filter)
         else:
@@ -172,6 +219,10 @@ class DatatableTable:
         if self.max_records:
             query = query[:self.max_records]
         query = self.extra_filters(query=query)
+        query = self.view_filter(query, self)
+        if aggregations:
+            # this returns a dictionary hence with have to put in a list
+            query = [query.aggregate(**aggregations)]
         return query
 
     def sort(self, *columns):
@@ -191,35 +242,47 @@ class DatatableTable:
                 c.setup_kwargs(options)
 
     def find_column(self, column_name, by_field=False):
+        if not by_field:
+            for n, c in enumerate(self.columns):
+                if c.column_name == column_name:
+                    return c, n
         for n, c in enumerate(self.columns):
-            if by_field and c.field == column_name:
+            if c.field == column_name:
                 return c, n
-            if c.column_name == column_name:
+        for n, c in enumerate(self.columns):
+            if c.column_name.split('/')[-1] == column_name:
                 return c, n
         raise DatatableError('Unable to find column ' + column_name)
 
     def add_columns(self, *columns):
         for c in columns:
-            self.columns += ColumnInitialisor(self.model, c).get_columns()
+            new_columns = ColumnInitialisor(self.model, c, table=self).get_columns()
+            self.columns += [nc for nc in new_columns if nc.enabled]
+        return self
 
     def fields(self):
-        fields = []
+
+        def add_fields(current_fields, new_fields):
+            if isinstance(new_fields, (tuple, list)):
+                current_fields.update(new_fields)
+            else:
+                current_fields.add(new_fields)
+
+        fields = set()
+        for p in self.get_result_processes():
+            if hasattr(p, 'field'):
+                add_fields(fields, p.field)
         for c in self.columns:
             if c.field and 'calculated' not in c.options:
-                if isinstance(c.field, (tuple, list)):
-                    for f in c.field:
-                        if f not in fields:
-                            fields.append(f)
-                else:
-                    if c.field not in fields:
-                        fields.append(c.field)
+                add_fields(fields, c.field)
         return fields
 
     def all_names(self):
         return [c.column_name for c in self.columns]
 
     def all_titles(self):
-        return [str(c.title) for c in self.columns]
+         return [mark_safe(str(c.title) + ('' if not c.popover else c.popover_html.format(c.popover)))
+                 for c in self.columns]
 
     def render(self):
         rendered_strings = []
@@ -243,6 +306,8 @@ class DatatableTable:
         options = dict(self.table_options)
         if not self.ajax_data:
             options['data'] = self.get_table_array(None, self.get_query())
+        if self.table_data:
+            options['data'] = self.get_table_array(None, self.table_data)
         options['columnDefs'] = [dict({'targets': i, 'name': c.column_name}, **c.style())
                                  for i, c in enumerate(self.columns)]
         table_vars = {
@@ -256,11 +321,27 @@ class DatatableTable:
         # col_def_str = col_def_str.replace('&"', "")
         return col_def_str
 
-    def get_table_array(self, request, results):
-        page_results = {}
+    def get_result_processes(self):
+        result_processes = {}
         for c in self.columns:
-            c.setup_results(request, page_results)
-        return [[c.row_result(data_dict, page_results) for c in self.columns] for data_dict in results]
+            result_processes.update(c.result_processes)
+        return result_processes.values()
+
+    def get_table_array(self, request, results):
+        result_processes = self.get_result_processes()
+        for p in result_processes:
+            p.setup_results(request, self.page_results)
+        for c in self.columns:
+            c.setup_results(request, self.page_results)
+        results_list = []
+        for data_dict in results:
+            try:
+                for p in result_processes:
+                    p.row_result(data_dict, self.page_results)
+                results_list.append([c.row_result(data_dict, self.page_results) for c in self.columns])
+            except DatatableExcludedRow:
+                pass
+        return results_list
 
     def get_json(self, request, results):
         return_data = {'data': self.get_table_array(request, results)}
@@ -284,34 +365,68 @@ class DatatableTable:
             del self.columns[self.find_column(n)[1]]
 
 
+class HorizontalTable(DatatableTable):
+
+    datatable_template = 'datatables/horizontal_table.html'
+
+    def __init__(self, *args, **kwargs):
+        pk = kwargs.pop('pk', None)
+        kwargs.setdefault('table_classes', ['table', 'table-sm', 'bg-light', 'w-100'])
+        super().__init__(*args, **kwargs)
+        if pk:
+            self.filter['pk'] = pk
+
+    def model_table_setup(self):
+        return mark_safe(json.dumps({'initsetup': json.loads(self.col_def_str()),
+                                     'data': self.get_table_array(self.kwargs.get('request'), self.get_query()),
+                                     'row_titles': self.all_titles(),
+                                     'table_id': self.table_id,
+                                     }))
+
+
 class DatatableView(TemplateView):
     model = None
     table_classes = None
     table_options = None
+    ajax_commands = ['row']
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.tables = {}
-        self.add_tables()
         self.dispatch_context = None
 
+    def row_edit(self, **kwargs):
+        row_data = json.loads(kwargs.pop('row_data'))
+        table = self.tables[kwargs['table_id']]
+        row_object = table.model.objects.get(pk=kwargs['row_no'][1:])
+        self.setup_table(table)
+        table.columns[kwargs['changed'][0]].alter_object(row_object, row_data[kwargs['changed'][0]])
+        return table.refresh_row(self.request, kwargs['row_no'])
+
+    def row_refresh(self, **kwargs):
+        table_id = kwargs.get('table_id', list(self.tables.keys())[0])
+        self.setup_table(self.tables[table_id])
+        return self.tables[table_id].refresh_row(self.request, kwargs['row_no'])
+
+    def view_filter(self, query, table):
+        if hasattr(table.model, 'query_filter'):
+            return table.model.query_filter(query, self.request, table=table)
+        return query
+
     def add_table(self, table_id, **kwargs):
         self.tables[table_id] = DatatableTable(table_id, table_options=self.table_options,
-                                               table_classes=self.table_classes,
-                                               **kwargs)
+                                               table_classes=self.table_classes, view=self, **kwargs)
 
     def add_tables(self):
         self.add_table(type(self).__name__.lower(), model=self.model)
 
     def setup_tables(self, table_id=None):
-        for t_id, table in self.tables.items():
-            if not table_id or t_id == table_id:
-                if t_id == type(self).__name__.lower():
-                    self.setup_table(table)
-                else:
-                    getattr(self, 'setup_' + t_id)(table)
+        for t_id, table in self.tables.items() if not table_id else [(table_id, self.tables[table_id])]:
+            getattr(self, 'setup_' + t_id, self.setup_table)(table)
+            table.view_filter = self.view_filter
 
     def dispatch(self, request, *args, **kwargs):
+        self.add_tables()
         self.dispatch_context = detect_device(request)
         return super(DatatableView, self).dispatch(request, *args, **kwargs)
 
@@ -391,3 +506,14 @@ class DatatableView(TemplateView):
 
     def add_to_context(self, **kwargs):
         return {}
+
+    def button_datatable_save_state(self, state, table_id, **kwargs):
+        saved_state = SavedState.objects.get_or_create(name=kwargs['name'], table_id=table_id)[0]
+        saved_state.state = json.dumps(state)
+        saved_state.save()
+        return self.command_response('delay', time=1)
+
+    def button_datatable_load_state(self, table_id, name, state_id, **kwargs):
+        saved_state = SavedState.objects.get(id=int(state_id))
+        self.add_command('restore_datatable', state=saved_state.state, table_id=table_id, state_id=saved_state.id)
+        return self.command_response('reload')
