@@ -1,6 +1,11 @@
+from importlib.metadata import PackageNotFoundError
+
 from ajax_helpers.html_include import SourceBase, pip_version
 
-version = pip_version('django-filtered-datatables')
+try:
+    version = pip_version('django-filtered-datatables')
+except PackageNotFoundError:
+    version = 'local'
 
 
 class DataTables(SourceBase):
@@ -14,6 +19,7 @@ class FilteredDataTables(SourceBase):
     static_path = 'django_datatables/'
     js_filename = 'filtered-datatable.js'
     css_filename = 'datatables.css'
+    legacy_js = ['runtime.js', 'filtered_datatables_legacy.js']
 
 
 class Moment(SourceBase):
@@ -31,5 +37,3 @@ class MomentDatatable(SourceBase):
 packages = {
     'datatable': [DataTables, FilteredDataTables, Moment, MomentDatatable],
 }
-
-
