@@ -1,13 +1,71 @@
 if (typeof django_datatables === 'undefined') {
     var django_datatables = function () {
-        var setup = {}
-        var DataTables = {}
+        var setup = {};
+        var DataTables = {};
 
         function b_r(button) {
-            var command = $(button).attr('data-command')
-            var row_id = $(button).closest('tr').attr('id')
-            var table_id = $(button).closest('table').attr('id')
-            DataTables[table_id].send_row(command, row_id)
+            var command = $(button).attr('data-command');
+            var row_id = $(button).closest('tr').attr('id');
+            var table_id = $(button).closest('table').attr('id');
+            DataTables[table_id].send_row(command, row_id);
+        }
+
+        function make_edit(span) {
+            var table_id = $(span).closest('table').attr('id');
+            var datatable = django_datatables.DataTables[table_id]
+            var cell = $(span).closest('td');
+            var cell_index = datatable.table.api().cell($(cell)).index();
+            var options = datatable.initsetup.colOptions[cell_index.column].edit_options
+            $(cell).html(datatable.initsetup.colOptions[cell_index.column].input_html)
+            var inp = $(cell).children()[0]
+            if (inp.nodeName == 'SELECT'){
+                $(inp).val(datatable.table.api().row(cell_index.row).data()[cell_index.column][0])
+                if (options !== null && options.select2){
+                    $(inp).select2(
+                            {minimumResultsForSearch: 10}
+                    );
+                    $(inp).on('select2:close', function (){
+                        django_datatables.row_send(inp)
+                    })
+                    $(inp).select2('open');
+                }
+            } else{
+                $(inp).val(datatable.table.api().row(cell_index.row).data()[cell_index.column])
+            }
+            $(inp).focus();
+        }
+
+        function row_send(button) {
+            var command = $(button).attr('data-command');
+            var row_id = $(button).closest('tr').attr('id');
+            var table_id = $(button).closest('table').attr('id');
+            var datatable = django_datatables.DataTables[table_id];
+            var row_data = datatable.table.api().row('#' + row_id).data();
+            var changed = [];
+            var row = $('#' + row_id)
+            $('input, select', row).each(function () {
+                var cell_index = datatable.table.api().cell($(this).closest('td')).index();
+                if (this.nodeName=='SELECT'){
+                    if (row_data[cell_index.column][0] != $(this).val()) {
+                        changed.push(cell_index.column);
+                        row_data[cell_index.column] = [$(this).val(), $("option:selected", $(this)).text()];
+                    }
+                } else if (this.nodeName=='INPUT'){
+                    if (row_data[cell_index.column] != $(this).val()) {
+                        changed.push(cell_index.column);
+                        row_data[cell_index.column] = $(this).val();
+                    }
+                }
+            });
+            if (changed.length) {
+                var data = {
+                    'row': 'edit', 'row_data': JSON.stringify(row_data), 'row_no': row_id, 'table_id': table_id ,
+                    'changed': changed
+                };
+                ajax_helpers.post_json({data: data});
+            } else {
+                django_datatables.DataTables[table_id].table.api().row('#' + row_id).invalidate();
+            }
         }
 
         ajax_helpers.command_functions.delete_row = function (command) {
@@ -15,13 +73,24 @@ if (typeof django_datatables === 'undefined') {
             DataTables[command.table_id].table.api().draw(false)
         }
         ajax_helpers.command_functions.refresh_row = function(command){
-            DataTables[command.table_id].table.api().row('#' + command.row_no).data(command.data).invalidate()
+            if (command.data === undefined){
+                ajax_helpers.post_json({data: {row: 'refresh', row_no: command.row_no, table_id: command.table_id}});
+            } else {
+                DataTables[command.table_id].table.api().row('#' + command.row_no).data(command.data).invalidate()
+            }
         }
 
         ajax_helpers.command_functions.reload_table = function(command){
             DataTables[command.table_id].table.api().ajax.reload(null, false);
         }
 
+        ajax_helpers.command_functions.restore_datatable = function (command) {
+            state = JSON.parse(command.state)
+            state.time = new Date().getTime()
+            state.state_id = command.state_id
+            localStorage.setItem('DataTables_' + command.table_id + '_' + location.pathname, JSON.stringify(state));
+        };
+
         var utilities = {
 
             numberWithCommas: function(x, decimal_places=0) {
@@ -181,7 +250,7 @@ if (typeof django_datatables === 'undefined') {
 
         FilterBase.prototype.restore_state = function (data) {
             try {
-                state_data = this.pTable.table.api().state.loaded().columns[this.column_no][this.storage_key]
+                var state_data = this.pTable.table.api().state.loaded().columns[this.column_no][this.storage_key]
             } catch (e) {
                 console.log(e)
                 return
@@ -288,7 +357,7 @@ if (typeof django_datatables === 'undefined') {
 
             this.html = function () {
                 var htmldata = ''
-                for (k of this.filter_calcs.sort_keys()) {
+                for (var k of this.filter_calcs.sort_keys()) {
                     htmldata += this.options.htmlcheckbox.replace(/%1/g, k).replace(/%6/g, encodeURI(k))
                 }
                 var context = $('#' + this.html_id)
@@ -415,6 +484,19 @@ if (typeof django_datatables === 'undefined') {
                 }
             },
 
+            Base64Row: function (column, params, table) {
+                django_datatables.BaseProcessAjaxData.call(this, column, params, table)
+                if (params.html === undefined) {
+                    this.convert = function (current, value, meta, row) {
+                        return current.replace(params.var, meta.settings.rowId(row))
+                    }
+                } else {
+                    this.convert = function (current, value, meta, row) {
+                        return params.html.replace(params.var, btoa(JSON.stringify(row)).replace(/\//g, '_').replace(/\+/g, '_'))
+                    }
+                }
+            },
+
             Replace: function (column, params, table) {
                 django_datatables.BaseProcessAjaxData.call(this, column, params, table)
                     if (Array.isArray(this.reg_exp) && table.initsetup.colOptions[column].field_array) {
@@ -490,7 +572,7 @@ if (typeof django_datatables === 'undefined') {
                 if (params.default){
                     this.convert = function (current, row) {
                         try {
-                            return_val = current.slice(0, current.length - 1).filter(function (l) {
+                            var return_val = current.slice(0, current.length - 1).filter(function (l) {
                                 return l != '' && l != null;
                             }).join(this.separator)
                         } catch (e) {
@@ -607,17 +689,17 @@ if (typeof django_datatables === 'undefined') {
                     {
                         dataSrc: tablesetup.field_ids.indexOf(tablesetup.tableOptions.rowGroup.dataSrc),
                         endRender: function (rows, group) {
-                            sums = Array(rows.data()[0].length).fill('')
+                            var sums = Array(rows.data()[0].length).fill('')
                             tablesetup.tableOptions.rowGroup.sumColumns.forEach(
                                 function (column) {
-                                    column_no = tablesetup.field_ids.indexOf(column)
+                                    var column_no = tablesetup.field_ids.indexOf(column)
                                     var sum = rows.data().pluck(column_no).reduce(
                                         function (a, b) {
                                             return a + parseFloat(b)
                                         }, 0)
                                     sums[column_no] = sum
                                 })
-                            sums_row = ''
+                            var sums_row = ''
                             for (c = 0; c < sums.length; c++) {
                                 if (tablesetup.colOptions[c].hidden != true) {
                                     if (typeof (sums[c]) == 'number') {
@@ -638,7 +720,7 @@ if (typeof django_datatables === 'undefined') {
 
             if (this.initsetup.tableOptions.row_href) {
                 $('#' + html_id + ' tbody').on('click', 'tr', function () {
-                    p_table = django_datatables.DataTables[html_id]
+                    var p_table = django_datatables.DataTables[html_id]
                     var row_id = $(this).attr('id')
                     var row_data = p_table.table.api().row('#' + row_id).data()
                     var href_render = new django_datatables.column_render(0, p_table.initsetup.tableOptions.row_href, p_table)
@@ -745,6 +827,8 @@ if (typeof django_datatables === 'undefined') {
             add_to_setup,
             b_r,
             PythonTable,
+            make_edit,
+            row_send,
         }
     }()
 }
@@ -761,4 +845,5 @@ function rep_options(html, option_dict) {
 
 $(document).ready(function () {
     $.fn.dataTable.moment("DD/MM/YYYY");
+    $.fn.dataTable.moment("DD/MM/YYYY HH:mm");
 });
